#ifndef BEHAVIOR_STATEMENT_H
#define BEHAVIOR_STATEMENT_H

#include "simplestatement.h"
#include "identifier.h"
#include "expressionlist.h"

class BehaviorStatement : public SimpleStatement {
	private:
		Identifier *behavior;
		Identifier *function;
		ExpressionList *expressionList;

	public:
		BehaviorStatement(SrcPosition *position, Identifier *behavior, Identifier *function, ExpressionList *expressionList)
				: SimpleStatement(position), behavior(behavior), function(function), expressionList(expressionList) {}

		BehaviorStatement(const BehaviorStatement& behaviorStatement) : SimpleStatement(behaviorStatement.position) {
			*(this) = behaviorStatement;
		}

		virtual ~BehaviorStatement() {
			delete this->expressionList;
		}

		ExpressionList *getExpressionList() {
			return this->expressionList;
		}

		Identifier *getBehavior() {
			return this->behavior;
		}
		
		Identifier *getFunction() {
			return this->function;
		}
		
		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitBehaviorStatement(this);
		}

		virtual Ast& clone() const {
			return *(new BehaviorStatement(new SrcPosition(*(this->position)),
					 this->behavior ? dynamic_cast<Identifier*>(&(this->behavior->clone())) : 0,
					 this->function ? dynamic_cast<Identifier*>(&(this->function->clone())) : 0,
					 this->expressionList ? dynamic_cast<ExpressionList*>(&(this->expressionList->clone())) : 0));
		}
};

#endif
