#ifndef FIELD_REFERENCE_H
#define FIELD_REFERENCE_H

#include "identifier.h"
#include "expression.h"
#include "leftside.h"

class FieldReference : public Expression, public LeftSide {

	private:
		Identifier *msgId;
		Identifier *fieldId;

	public:
		FieldReference(SrcPosition *position, Identifier *msgId, Identifier *fieldId)
				: Expression(position), LeftSide(position), Ast(position), msgId(msgId), fieldId(fieldId) {}
		
		FieldReference(const FieldReference& fieldReference) : Expression(fieldReference.position), LeftSide(fieldReference.position), Ast(position) {
			(*this) = fieldReference;
		}

		virtual ~FieldReference() {
			delete this->msgId;
			delete this->fieldId;
		}

		Identifier *getMsgId() {
			return this->msgId;
		}

		Identifier *getFieldId() {
			return this->fieldId;
		}

		virtual Ast& operator = (const Ast& ast);

		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitFieldReference(this);
		}

		virtual Ast& clone() const {
			return *(new FieldReference(new SrcPosition(*(this->position)),
							this->msgId ? dynamic_cast<Identifier*>(&(this->msgId->clone())): 0,
							this->fieldId ? dynamic_cast<Identifier*>(&(this->fieldId->clone())): 0));
		}

};

#endif
