#ifndef FIELD_TYPE_H
#define FIELD_TYPE_H

#include "ast.h"
#include "expression.h"

class FieldType : public Ast {
	public:
		enum FType {BIT, BYTE};
		
		FieldType(SrcPosition *position, Expression *expression, FType type)
				: Ast(position), expression(expression), type(type) {}

		FieldType(const FieldType& fieldType) : Ast(fieldType.position) {
			*(this) = fieldType;
		}

		virtual ~FieldType() {
			delete this->expression;
		}

		Expression *getExpression() {
			return this->expression;
		}

		FType getType() const {
			return this->type;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitFieldType(this);
		}

		virtual Ast& clone() const {
			return *(new FieldType(new SrcPosition(*(this->position)),
					 this->expression ? dynamic_cast<Expression*>(&(this->expression->clone())) : 0,
					 this->type));
		}
	 private:
		Expression *expression;
		FType type;
};

#endif
