#ifndef SEND_STATEMENT_H
#define SEND_STATEMENT_H

#include "simplestatement.h"
#include "identifier.h"

class SendStatement : public SimpleStatement {
	private:
		Identifier *msgId;
		Identifier *addrId;
		bool tcp;

	public:
		SendStatement(SrcPosition *position, Identifier *msgId, Identifier *addrId, bool tcp)
				: SimpleStatement(position), msgId(msgId), addrId(addrId), tcp(tcp) {}

		SendStatement(const SendStatement& sendStatement) : SimpleStatement(sendStatement.position){
			(*this) = sendStatement;
		}

		virtual ~SendStatement() {
			delete this->msgId;
			delete this->addrId;
		}

		Identifier *getMsgId() {
			return this->msgId;
		}

		Identifier *getAddrId() {
			return this->addrId;
		}

		bool isTcp() const {
			return this->tcp;
		}

		virtual Ast& operator = (const Ast& ast);

		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSendStatement(this);
		}

		virtual Ast& clone() const {
			return *(new SendStatement(new SrcPosition(*this->position),
							this->msgId ? dynamic_cast<Identifier*>(&(this->msgId->clone())): 0,
							this->addrId ? dynamic_cast<Identifier*>(&(this->addrId->clone())): 0,
							this->tcp));
		}
};

#endif
