#ifndef SEQUENTIAL_BEHAVIOR_H
#define SEQUENTIAL_BEHAVIOR_H

#include "behavior.h"
#include "simplebehavior.h"

class SequentialBehavior : public Behavior {
	private:
		Behavior *behavior;
		SimpleBehavior *simpleBehavior;


	public:
		SequentialBehavior(SrcPosition *position, Behavior *behavior, SimpleBehavior *simpleBehavior)
			: Behavior(position), behavior(behavior), simpleBehavior(simpleBehavior) {}

		SequentialBehavior(const SequentialBehavior& sequentialBehavior) : Behavior(sequentialBehavior.position) {
			*(this) = sequentialBehavior;
		}

		virtual ~SequentialBehavior() {
			delete this->behavior;
			delete this->simpleBehavior;
		}

		Behavior *getBehavior() {
			return this->behavior;
		}

		SimpleBehavior *getSimpleBehavior() {
			return this->simpleBehavior;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSequentialBehavior(this);
		}

		virtual Ast& clone() const {
			return *(new SequentialBehavior(new SrcPosition(*(this->position)),
					 this->behavior ? dynamic_cast<Behavior*>(&(this->behavior->clone())) : 0,
					 this->simpleBehavior ? dynamic_cast<SimpleBehavior*>(&(this->simpleBehavior->clone())) : 0));
		}
};

#endif
