#ifndef SEQUENTIAL_DECLARATION_H
#define SEQUENTIAL_DECLARATION_H

#include "declaration.h"
#include "simpledeclaration.h"

class SequentialDeclaration : public Declaration {
	private:
		Declaration *declaration;
		SimpleDeclaration *simpleDeclaration;


	public:
		SequentialDeclaration(SrcPosition *position, Declaration *declaration, SimpleDeclaration *simpleDeclaration)
			: Declaration(position), declaration(declaration), simpleDeclaration(simpleDeclaration) {}

		SequentialDeclaration(const SequentialDeclaration& sequentialDeclaration) : Declaration(sequentialDeclaration.position) {
			*(this) = sequentialDeclaration;
		}

		virtual ~SequentialDeclaration() {
			delete this->declaration;
			delete this->simpleDeclaration;
		}

		Declaration *getDeclaration() {
			return this->declaration;
		}

		SimpleDeclaration *getSimpleDeclaration() {
			return this->simpleDeclaration;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSequentialDeclaration(this);
		}

		virtual Ast& clone() const {
			return *(new SequentialDeclaration(new SrcPosition(*(this->position)),
					 this->declaration ? dynamic_cast<Declaration*>(&(this->declaration->clone())) : 0,
					 this->simpleDeclaration ? dynamic_cast<SimpleDeclaration*>(&(this->simpleDeclaration->clone())) : 0));
		}
};

#endif
