#ifndef SEQUENTIAL_PROCESS_H
#define SEQUENTIAL_PROCESS_H

#include "process.h"
#include "simpleprocess.h"

class SequentialProcess : public Process {
	private:
		Process *process;
		SimpleProcess *simpleProcess;


	public:
		SequentialProcess(SrcPosition *position, Process *process, SimpleProcess *simpleProcess)
			: Process(position), process(process), simpleProcess(simpleProcess) {}

		SequentialProcess(const SequentialProcess& sequentialProcess) : Process(sequentialProcess.position) {
			*(this) = sequentialProcess;
		}

		virtual ~SequentialProcess() {
			delete this->process;
			delete this->simpleProcess;
		}

		Process *getProcess() {
			return this->process;
		}

		SimpleProcess *getSimpleProcess() {
			return this->simpleProcess;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSequentialProcess(this);
		}

		virtual Ast& clone() const {
			return *(new SequentialProcess(new SrcPosition(*(this->position)),
					 this->process ? dynamic_cast<Process*>(&(this->process->clone())) : 0,
					 this->simpleProcess ? dynamic_cast<SimpleProcess*>(&(this->simpleProcess->clone())) : 0));
		}
};

#endif
