#ifndef SIMPLE_FIELD_DECLARATION_H
#define SIMPLE_FIELD_DECLARATION_H

#include "fielddeclaration.h"
#include "identifier.h"
#include "fieldtype.h"
#include "expression.h"
#include "stringliteral.h"

class SimpleFieldDeclaration : public FieldDeclaration {
	private:
		Identifier *identifier;
		FieldType *fieldType;
		Expression *expression;
		StringLiteral *stringLiteral;

	public:
		SimpleFieldDeclaration(SrcPosition *position, Identifier *identifier, FieldType *fieldType, Expression *expression, StringLiteral *stringLiteral)
			: FieldDeclaration(position), identifier(identifier), fieldType(fieldType), expression(expression), stringLiteral(stringLiteral) {}

		SimpleFieldDeclaration(const SimpleFieldDeclaration& simpleFieldDeclaration) : FieldDeclaration(simpleFieldDeclaration.position) {
			*(this) = simpleFieldDeclaration;
		}

		virtual ~SimpleFieldDeclaration() {
			delete this->identifier;
			delete this->fieldType;
			delete this->expression;
			delete this->stringLiteral;
		}

		Identifier *getIdentifier() {
			return this->identifier;
		}

		FieldType *getFieldType() {
			return this->fieldType;
		}

		Expression *getExpression() {
			return this->expression;
		}

		StringLiteral *getStringLiteral() {
			return this->stringLiteral;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleFieldDeclaration(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleFieldDeclaration(new SrcPosition(*(this->position)),
					 this->identifier ? dynamic_cast<Identifier*>(&(this->identifier->clone())) : 0,
					 this->fieldType ? dynamic_cast<FieldType*>(&(this->fieldType->clone())) : 0,
					 this->expression ? dynamic_cast<Expression*>(&(this->expression->clone())) : 0,
					 this->stringLiteral ? dynamic_cast<StringLiteral*>(&(this->stringLiteral->clone())) : 0));
		}
};

#endif
