#ifndef SIMPLE_GUARDED_STATEMENT_H
#define SIMPLE_GUARDED_STATEMENT_H

#include "guardedstatement.h"
#include "expression.h"
#include "statement.h"

class SimpleGuardedStatement : public GuardedStatement {
	private:
		Expression *expression;
		Statement *statement;

	public:
		SimpleGuardedStatement(SrcPosition *position, Expression *expression, Statement *statement)
			: GuardedStatement(position), expression(expression), statement(statement) {}

		SimpleGuardedStatement(const SimpleGuardedStatement& simpleGuardedStatement) : GuardedStatement(simpleGuardedStatement.position) {
			*(this) = simpleGuardedStatement;
		}

		virtual ~SimpleGuardedStatement() {
			delete this->expression;
			delete this->statement;
		}

		Expression *getExpression() {
			return this->expression;
		}

		Statement *getStatement() {
			return this->statement;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleGuardedStatement(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleGuardedStatement(new SrcPosition(*(this->position)),
					 this->expression ? dynamic_cast<Expression*>(&(this->expression->clone())) : 0,
					 this->statement ? dynamic_cast<Statement*>(&(this->statement->clone())) : 0));
		}
};

#endif
