#ifndef SIMPLE_MESSAGE_H
#define SIMPLE_MESSAGE_H

#include "message.h"
#include "identifier.h"
#include "fielddeclaration.h"

class SimpleMessage : public Message {
	private:
		Identifier *identifier;
		FieldDeclaration *fieldDeclaration;
		Identifier *handlerIn;
		Identifier *handlerOut;
		bool external;

	public:
		SimpleMessage(SrcPosition *position, Identifier *identifier, FieldDeclaration *fieldDeclaration, 
					  Identifier *handlerIn, Identifier *handlerOut, bool external)
			: Message(position), identifier(identifier), fieldDeclaration(fieldDeclaration), 
			  handlerIn(handlerIn), handlerOut(handlerOut), external(external) {}

		SimpleMessage(const SimpleMessage& simpleMessage) : Message(simpleMessage.position) {
			*(this) = simpleMessage;
		}

		virtual ~SimpleMessage() {
			delete this->identifier;
			delete this->fieldDeclaration;
			delete this->handlerIn;
			delete this->handlerOut;
		}

		Identifier *getIdentifier() {
			return this->identifier;
		}

		FieldDeclaration *getFieldDeclaration() {
			return this->fieldDeclaration;
		}

		Identifier *getHandlerIn() {
			return this->handlerIn;
		}

		Identifier *getHandlerOut() {
			return this->handlerOut;
		}

		bool isExternal() const {
			return this->external;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitSimpleMessage(this);
		}

		virtual Ast& clone() const {
			return *(new SimpleMessage(new SrcPosition(*(this->position)),
					 this->identifier ? dynamic_cast<Identifier*>(&(this->identifier->clone())) : 0,
					 this->fieldDeclaration ? dynamic_cast<FieldDeclaration*>(&(this->fieldDeclaration->clone())) : 0,
					 this->handlerIn ? dynamic_cast<Identifier*>(&(this->handlerIn->clone())) : 0,
					 this->handlerOut ? dynamic_cast<Identifier*>(&(this->handlerOut->clone())) : 0,
					 this->external));
		}
};

#endif
