#include "typecheckervisitor.h"
#include "protomake.h"
#include <typeinfo>
#include <string>
#include <iostream>

#define GLOBAL "&GLOBAL&"

using namespace std;

VisitorReturn *TypeCheckerVisitor::visitAction(Action *action) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitActStatement(ActStatement *actStatement) {
	VisitorReturn *v= new VisitorReturn;
	Type *t = new Type(actStatement->getIdentifier()->getPosition(), Type::TIMEOUT);
	string str = actStatement->getIdentifier()->getName();
	
	SymbolTableType::iterator i = this->symbolTable[currentProcess].find("@" + str);
	if(i != this->symbolTable[currentProcess].end()) {
		t = this->symbolTable[currentProcess]["@" + str];
		this->symbolTable[currentProcess].erase("@" + str);
	}
	
	this->stack.push_back(t);
	this->insert = true;
	v = actStatement->getIdentifier()->visit(*this);
	this->insert = false;
	this->stack.pop_back();
	
	v = actStatement->getExpression()->visit(*this);
	if(v->type->getTypeCode() != Type::INTEGER) {
		cerr << "linha " << actStatement-> << " Expression no  INTEGER\n";
		exit(1);
	}
	
	return v; // essse return nao serve pra nada
}

VisitorReturn *TypeCheckerVisitor::visitArrayReference(ArrayReference *arrayReference) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitArrayType(ArrayType *arrayType) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitAssignStatement(AssignStatement *assignStatement) {
	VisitorReturn *v1, *v2;

	v1 = assignStatement->getLeftSideList()->visit(*this);
	v2 = assignStatement->getExpressionList()->visit(*this);
	
	if(v1->type->getTypeCode() != v2->type->getTypeCode()) {
		cerr << "Tipos de expressoes no so iguais\n";
		exit(1);
	}
		
	return v1; // essse return nao serve pra nada
}

VisitorReturn *TypeCheckerVisitor::visitAst(Ast *ast) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitBehavior(Behavior *behavior) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitBehaviorStatement(BehaviorStatement *behaviorStatement) {
	VisitorReturn *v= new VisitorReturn;
	
	v = behaviorStatement->getBehavior()->visit(*this);
	if(v->type->getTypeCode() != Type::BEHAVIOR) {
		cerr << "Behavior no  BEHAVIOR\n";
		exit(1);
	}
	
	if(behaviorStatement->getExpressionList() != NULL)
		behaviorStatement->getExpressionList()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitBinaryExpression(BinaryExpression *binaryExpression) {
	VisitorReturn *v1, *v2;
	string op;
	Type *type;

	v1 = binaryExpression->getLeftExpression()->visit(*this);
	v2 = binaryExpression->getRightExpression()->visit(*this);
	op = binaryExpression->getOp();
	
	if(v1->type->getTypeCode() != v2->type->getTypeCode()) {
		cerr << "Tipos de expresses binrias no so iguais\n";
		exit(1);
	}
	
	if((op == "&") || (op == "|")) {
		type = new Type(binaryExpression->getPosition(),Type::BOOLEAN);
		if(v1->type->getTypeCode() != Type::BOOLEAN) {
			cerr << "Operadores booleanos aplicados a expresses no booleanas\n";
			exit(1);
		}
	} else {
		if((op == "+") || (op == "-") || (op == "*") || (op == "/")) {
			type = new Type(binaryExpression->getPosition(),Type::INTEGER);
		} else {
			type = new Type(binaryExpression->getPosition(),Type::BOOLEAN);
		}
		if((v1->type->getTypeCode() != Type::INTEGER) && (op != "=")) {
			cerr << "Operadores aritmeticos aplicados a expresses no inteiras\n";
			exit(1);
		}
	}
	
	v1->type = type;
		
	return v1; 
}

VisitorReturn *TypeCheckerVisitor::visitDeclaration(Declaration *declaration) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitDoStatement(DoStatement *doStatement) {
	VisitorReturn *v= new VisitorReturn;
	
	v = doStatement->getGuardedStatement()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitExpression(Expression *expression) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitExpressionList(ExpressionList *expressionList) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitFieldDeclaration(FieldDeclaration *fieldDeclaration) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitFieldReference(FieldReference *fieldReference) {
	VisitorReturn *v= new VisitorReturn;
	string id;
	
	id = fieldReference->getMsgId()->getName() + "." + fieldReference->getFieldId()->getName();
	SymbolTableType::iterator i = this->symbolTable[GLOBAL].find(id);
	if(i == this->symbolTable[GLOBAL].end()) {
		cerr << "FieldReference no existe\n";
		exit(1);
	}
	
	v->type = this->symbolTable[GLOBAL][id];
	
	return v; 
}

VisitorReturn *TypeCheckerVisitor::visitFieldType(FieldType *fieldType) {
	VisitorReturn *v= new VisitorReturn;
	SimpleMessage *sm;
	Identifier *id;
	string str;
	bool ehInteger = true;
	IntegerLiteral *il = 0;
	
	if(typeid(*fieldType->getExpression()) == typeid(Identifier)) {
		ehInteger = false;
		sm = (SimpleMessage *)this->stack[this->stack.size()-1];
		id = (Identifier *)fieldType->getExpression();
		str = sm->getIdentifier()->getName() + "." + id->getName();
		
		SymbolTableType::iterator i = this->symbolTable[GLOBAL].find(str);
		if(i == this->symbolTable[GLOBAL].end()) {
			cerr << "FieldReference no existe\n";
			exit(1);
		}
		Type *t = (Type *)this->symbolTable[GLOBAL][str];
		if(t->getTypeCode()==Type::STRING) {
			cerr << "Quantificador invalido, tipo bit requerido\n";
			exit(1);
		}
		
	} else if(typeid(*fieldType->getExpression()) != typeid(IntegerLiteral)) {
		cerr << "Quantificador invalido, constante ou nome de campo requerido\n";
		exit(1);
	} else {
		il = (IntegerLiteral *)fieldType->getExpression();
	}
	
	if(fieldType->getType() == FieldType::BIT && !ehInteger) {
		cerr << "Se for bit s pode ser constante\n";
		exit(1);
	} else if(ehInteger && (il->getValue() > 32) && (fieldType->getType() == FieldType::BIT)) {
		cerr << "So pode ter no maximo 32 bits\n";
		exit(1);
	}
	
	Type *type = new Type(fieldType->getPosition(), fieldType->getType() == FieldType::BIT ? Type::INTEGER : Type::STRING);
	v->type = type;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitGuard(Guard *guard) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitGuardedStatement(GuardedStatement *guardedStatement) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitGuardExpression(GuardExpression *guardExpression) {
	VisitorReturn *v= new VisitorReturn;
	
	v = guardExpression->getExpression()->visit(*this);
	if(v->type->getTypeCode() != Type::BOOLEAN) {
		cerr << "A guarda tem que ser booleana\n";
		exit(1);
	}
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitGuardReceive(GuardReceive *guardReceive) {
	VisitorReturn *v= new VisitorReturn;
	guardReceive->getReceiveStatement()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitGuardTimeout(GuardTimeout *guardTimeout) {
	VisitorReturn *v= new VisitorReturn;
	
	this->forgetIt = true;
	v = guardTimeout->getIdentifier()->visit(*this);
	this->forgetIt = false;
	
	if(v->type->getTypeCode() != Type::TIMEOUT) {
		cerr << "O identifier tem que ser um timeout\n";
		exit(1);
	}
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitIdentifier(Identifier *identifier) {
	VisitorReturn *v = new VisitorReturn;
	Type *t = 0;
	
	if(this->insert) {
		t = (Type *)this->stack[this->stack.size()-1];
		SymbolTableType::iterator i = this->symbolTable[currentProcess].find(identifier->getName());
		if(i != this->symbolTable[currentProcess].end()) {
			cerr << "identifier jah declarado\n";
			exit(1);
		}
		this->symbolTable[currentProcess][identifier->getName()] = t;
		v->type = t;
	} else {
		string str = identifier->getName();
		
		SymbolTableType::iterator i = this->symbolTable[GLOBAL].find(str);
		//PAREI AQUI!
		if(i == this->symbolTable[GLOBAL].end()) {
			 i = this->symbolTable[currentProcess].find(str);
			 if(i == this->symbolTable[currentProcess].end()) {
				 if(!this->forgetIt) {
					 cerr << "identifier nao declarado\n";
					 exit(1);
				 } else {
					 str = "@" + str;
					 this->symbolTable[currentProcess][str] = new Type(identifier->getPosition(), Type::TIMEOUT);
				 }
			 }
			 t = this->symbolTable[currentProcess][str];
			 v->type = t;
		} else {
			 t = this->symbolTable[GLOBAL][str];
			 v->type = t;
		}
	}
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitIdentifierList(IdentifierList *identifierList) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitIfStatement(IfStatement *ifStatement) {
	VisitorReturn *v= new VisitorReturn;
	ifStatement->getGuardedStatement()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitImport(Import *import) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitLeftSide(LeftSide *leftSide) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitLeftSideList(LeftSideList *leftSideList) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitMessage(Message *message) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitMultipleArrayReference(MultipleArrayReference *multipleArrayReference) {
	VisitorReturn *v= new VisitorReturn;
	v = multipleArrayReference->getArrayReference()->visit(*this);
	if(v->type->getTypeCode() != Type::ARRAY) {
		cerr << "A referencia do array deve ser um ARRAY\n";
		exit(1);
	}
	v = multipleArrayReference->getExpression()->visit(*this);
	if(v->type->getTypeCode() != Type::INTEGER) {
		cerr << "A expression do array deve ser um INTEGER\n";
		exit(1);
	}
	Type *type = new Type(multipleArrayReference->getPosition(), Type::ARRAY);
	v->type = type;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitProcess(Process *process) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitProgram(Program *program) {
	VisitorReturn *v= new VisitorReturn;
	if(program->getImport())
		program->getImport()->visit(*this);
	if(program->getBehavior())
		program->getBehavior()->visit(*this);

	program->getMessage()->visit(*this);
	program->getProcess()->visit(*this);


	ProcessTableType::iterator i = this->symbolTable.begin();
	bool mainExists = false;

	while(i != this->symbolTable.end()) {
		if((*i).first[0] == '@') {
			cerr << "Existem identificadores invalidos\n";
			exit(1);
		}
		
		SymbolTableType::iterator j = (*i).second.begin();
		while(j != (*i).second.end()) {
			if((*j).first[0] == '@') {
				cerr << "Existem identificadores invalidos\n";
				exit(1);
			}	
			j++;
		}
		
		if(!mainExists && ((*i).first == "main")) {
			mainExists = true;
		}
		i++;
	}

	if(!mainExists) {
		cerr << "Tem que haver um processo chamado \"main\"\n";
		exit(1);
	}
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitRangeType(RangeType *rangeType) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitReceiveStatement(ReceiveStatement *receiveStatement) {
	VisitorReturn *v= new VisitorReturn;
	v = receiveStatement->getMsgId()->visit(*this);
	if(v->type->getTypeCode() != Type::MESSAGE) {
		cerr << "O identifier no representa uma mensagem\n";
		exit(1);
	}
	v = receiveStatement->getAddrId()->visit(*this);
	if((v->type->getTypeCode() != Type::ADDRESS) && 
	   (v->type->getTypeCode() != Type::CLIENT_ADDRESS) &&
	   (v->type->getTypeCode() != Type::SERVER_ADDRESS)) {
		cerr << "O identifier no representa um ADDRESS\n";
		exit(1);
	}
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSendStatement(SendStatement *sendStatement) {
	VisitorReturn *v= new VisitorReturn;
	v = sendStatement->getMsgId()->visit(*this);
	if(v->type->getTypeCode() != Type::MESSAGE) {
		cerr << "O identifier no representa uma mensagem\n";
		exit(1);
	}
	v = sendStatement->getAddrId()->visit(*this);
	if((v->type->getTypeCode() != Type::ADDRESS) && 
	   (v->type->getTypeCode() != Type::CLIENT_ADDRESS) &&
	   (v->type->getTypeCode() != Type::SERVER_ADDRESS)) {
		cerr << "O identifier no representa um ADDRESS\n";
		exit(1);
	}
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialAction(SequentialAction *sequentialAction) {
	VisitorReturn *v= new VisitorReturn;
	sequentialAction->getAction()->visit(*this);
	sequentialAction->getSimpleAction()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialBehavior(SequentialBehavior *sequentialBehavior) {
	VisitorReturn *v= new VisitorReturn;
	sequentialBehavior->getBehavior()->visit(*this);
	sequentialBehavior->getSimpleBehavior()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialDeclaration(SequentialDeclaration *sequentialDeclaration) {
	VisitorReturn *v= new VisitorReturn;
   
	if(sequentialDeclaration->getDeclaration())
		sequentialDeclaration->getDeclaration()->visit(*this);
	if(sequentialDeclaration->getSimpleDeclaration())
		sequentialDeclaration->getSimpleDeclaration()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialExpressionList(SequentialExpressionList *sequentialExpressionList) {
	VisitorReturn *v= new VisitorReturn;
   
	v = sequentialExpressionList->getExpressionList()->visit(*this);
	this->stack.push_back(v->type);
	v = sequentialExpressionList->getSimpleExpressionList()->visit(*this);
	this->stack.push_back(v->type);
			
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialFieldDeclaration(SequentialFieldDeclaration *sequentialFieldDeclaration) {
	VisitorReturn *v= new VisitorReturn;
   
	sequentialFieldDeclaration->getFieldDeclaration()->visit(*this);
 	sequentialFieldDeclaration->getSimpleFieldDeclaration()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialGuardedStatement(SequentialGuardedStatement *sequentialGuardedStatement) {
	VisitorReturn *v= new VisitorReturn;
   
	sequentialGuardedStatement->getGuardedStatement()->visit(*this);
	sequentialGuardedStatement->getSimpleGuardedStatement()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialIdentifierList(SequentialIdentifierList *sequentialIdentifierList) {
	VisitorReturn *v= new VisitorReturn;
   
	sequentialIdentifierList->getIdentifierList()->visit(*this);
	sequentialIdentifierList->getSimpleIdentifierList()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialImport(SequentialImport *sequentialImport) {
	VisitorReturn *v= new VisitorReturn;
   
	sequentialImport->getImport()->visit(*this);
	sequentialImport->getSimpleImport()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialLeftSideList(SequentialLeftSideList *sequentialLeftSideList) {
	VisitorReturn *v= new VisitorReturn;
   
	v = sequentialLeftSideList->getLeftSideList()->visit(*this);
	this->stack.push_back(v->type);
	v = sequentialLeftSideList->getSimpleLeftSideList()->visit(*this);
	this->stack.push_back(v->type);

	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialMessage(SequentialMessage *sequentialMessage) {
	VisitorReturn *v= new VisitorReturn;
   sequentialMessage->getMessage()->visit(*this);
	sequentialMessage->getSimpleMessage()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialProcess(SequentialProcess *sequentialProcess) {
	VisitorReturn *v= new VisitorReturn;
   sequentialProcess->getProcess()->visit(*this);
	sequentialProcess->getSimpleProcess()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSequentialStatement(SequentialStatement *sequentialStatement) {
	VisitorReturn *v= new VisitorReturn;
   
	sequentialStatement->getStatement()->visit(*this);
	sequentialStatement->getSimpleStatement()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleAction(SimpleAction *simpleAction) {
	VisitorReturn *v= new VisitorReturn;
	
	simpleAction->getGuard()->visit(*this);
	simpleAction->getStatement()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleArrayReference(SimpleArrayReference *simpleArrayReference) {
	VisitorReturn *v1, *v2;
	
	v1 = simpleArrayReference->getIdentifier()->visit(*this);
	if(v1->type->getTypeCode() != Type::ARRAY) {
		cerr << "Identifier tem que ser ARRAY\n";
		exit(1);
	}
	v2 = simpleArrayReference->getExpression()->visit(*this);
	if(v2->type->getTypeCode() != Type::INTEGER) {
		cerr << "Expressao tem que ser INTEGER\n";
		exit(1);
	}
	ArrayType *arrayType = (ArrayType*)v1->type;
	
	Type *type = new Type(simpleArrayReference->getPosition(), arrayType->getType()->getTypeCode());
	v1->type = type;
	
	return v1;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleBehavior(SimpleBehavior *simpleBehavior) {
	VisitorReturn *v= new VisitorReturn;
	Type *type = new Type(simpleBehavior->getPosition(), Type::BEHAVIOR);
	
	this->stack.push_back(type);
	string oldProcess = currentProcess;
	currentProcess = GLOBAL;
	this->insert = true;
	simpleBehavior->getIdentifierList()->visit(*this);
	this->stack.pop_back();
	this->insert = false;
	currentProcess = oldProcess;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleDeclaration(SimpleDeclaration *simpleDeclaration) {
	VisitorReturn *v1, *v2;
	
	this->stack.push_back(simpleDeclaration->getType());
	this->insert = true;
	
	simpleDeclaration->getIdentifierList()->visit(*this);
	this->stack.pop_back();
	this->insert = false;
	
	
	v2 = simpleDeclaration->getType()->visit(*this);
	if(simpleDeclaration->getExpression()) {
		v1 = simpleDeclaration->getExpression()->visit(*this);

		if(v1->type->getTypeCode() != v2->type->getTypeCode()) {
			cerr << "Declaracao e expressao tem tipos diferentes\n";
			exit(1);
		}
	}
	
	return v2; // esse retormo nao servepra nada
}

VisitorReturn *TypeCheckerVisitor::visitSimpleExpressionList(SimpleExpressionList *simpleExpressionList) {
	VisitorReturn *v= new VisitorReturn;
	v = simpleExpressionList->getExpression()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleFieldDeclaration(SimpleFieldDeclaration *simpleFieldDeclaration) {
	VisitorReturn *v1, *v2;
	SimpleMessage *sm;
	string str;
	
	
	v2 = simpleFieldDeclaration->getFieldType()->visit(*this);
	
	if(simpleFieldDeclaration->getExpression()) {
		v1 = simpleFieldDeclaration->getExpression()->visit(*this);
	
		if(v1->type->getTypeCode() != v2->type->getTypeCode()) {
			cerr << "Declaracao e expressao tem tipos diferentes\n";
			exit(1);
		}
	}
	
	sm = (SimpleMessage *)this->stack[this->stack.size()-1];
	str = sm->getIdentifier()->getName() + "." + simpleFieldDeclaration->getIdentifier()->getName();
	
	SymbolTableType::iterator i = this->symbolTable[GLOBAL].find(str);
	if(i != this->symbolTable[GLOBAL].end()) {
		cerr << "Nome do campo j existe\n";
		exit(1);
	}
	
	this->symbolTable[GLOBAL][str] = v2->type;
	
	return v2; // esse retormo nao servepra nada
}

VisitorReturn *TypeCheckerVisitor::visitSimpleGuardedStatement(SimpleGuardedStatement *simpleGuardedStatement) {
	VisitorReturn *v= new VisitorReturn;
	
	// \_:D_/
	simpleGuardedStatement->getStatement()->visit(*this);
	
	v = simpleGuardedStatement->getExpression()->visit(*this);
	if(v->type->getTypeCode() != Type::BOOLEAN) {
		cerr << "A guarda tem que conter uma expressao booleana \\_:D_/\n";
		exit(1);
	}
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleIdentifierList(SimpleIdentifierList *simpleIdentifierList) {
	VisitorReturn *v= new VisitorReturn;
	simpleIdentifierList->getIdentifier()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleImport(SimpleImport *simpleImport) {
	VisitorReturn *v= new VisitorReturn; // \_:)_/   \_:D_/   
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleLeftSideList(SimpleLeftSideList *simpleLeftSideList) {
	VisitorReturn *v= new VisitorReturn;
	v = simpleLeftSideList->getLeftSide()->visit(*this);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleMessage(SimpleMessage *simpleMessage) {
	VisitorReturn *v= new VisitorReturn;
	Type *type = new Type(simpleMessage->getPosition(), Type::MESSAGE);
	
	this->stack.push_back(simpleMessage);
	this->stack.push_back(type);
	currentProcess = GLOBAL;
	this->insert = true;
	simpleMessage->getIdentifier()->visit(*this);
	this->stack.pop_back();
	this->insert = false;
	
	simpleMessage->getFieldDeclaration()->visit(*this);
	this->stack.pop_back();
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleProcess(SimpleProcess *simpleProcess) {
	VisitorReturn *v= new VisitorReturn;
	Type *type = new Type(simpleProcess->getPosition(), Type::PROCESS);

	this->stack.push_back(type);
	this->insert = true;
	currentProcess = GLOBAL;
	v = simpleProcess->getIdentifier()->visit(*this);
	this->stack.pop_back();
	this->insert = false;
	
	string str = "@";
	
	str += simpleProcess->getIdentifier()->getName();
	SymbolTableType::iterator i = this->symbolTable[GLOBAL].find(str);
	if(i != this->symbolTable[GLOBAL].end()) {
		this->symbolTable[GLOBAL].erase(str);
	}
	
	this->currentProcess = simpleProcess->getIdentifier()->getName();
	
	simpleProcess->getDeclaration()->visit(*this);
	simpleProcess->getAction()->visit(*this);
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSimpleStatement(SimpleStatement *simpleStatement) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitSkipStatement(SkipStatement *skipStatement) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitStartStatement(StartStatement *startStatement) {
	VisitorReturn *v= new VisitorReturn;
	string str = "@";
	
	str += startStatement->getIdentifier()->getName();
	SymbolTableType::iterator i = this->symbolTable[GLOBAL].find(str);
	
	//soh pra criar a tabela atraves do primeiro acesso.
	//this->symbolTable[str].begin();
	
	if(i == this->symbolTable[GLOBAL].end()) {
		this->symbolTable[GLOBAL][str] = new Type(startStatement->getPosition(),Type::PROCESS);
	} else {
		v = startStatement->getIdentifier()->visit(*this);
		if(v->type->getTypeCode() != Type::PROCESS) {
			cerr << "Identificador tem que ser um processo \\_:D_/\n";
			exit(1);
		}
	}
	
	
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitStatement(Statement *statement) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitStopStatement(StopStatement *stopStatement) {
	VisitorReturn *v= new VisitorReturn;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitStringLiteral(StringLiteral *stringLiteral) {
	VisitorReturn *v= new VisitorReturn;
	v->type = new Type(stringLiteral->getPosition(),Type::STRING);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitIntegerLiteral(IntegerLiteral *integerLiteral) {
	VisitorReturn *v= new VisitorReturn;
	v->type = new Type(integerLiteral->getPosition(),Type::INTEGER);
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitType(Type *type) {
	VisitorReturn *v= new VisitorReturn;
	v->type = type;
	return v;
}

VisitorReturn *TypeCheckerVisitor::visitUnaryExpression(UnaryExpression *unaryExpression) {
	VisitorReturn *v= new VisitorReturn;
	string op;

	v = unaryExpression->getExpression()->visit(*this);
	op = unaryExpression->getOp();
	
	if(op == "~") {
		if(v->type->getTypeCode() != Type::BOOLEAN) {
			cerr << "Operadores booleanos aplicados a expresses no booleanas\n";
			exit(1);
		}
	} else {
		if(v->type->getTypeCode() != Type::INTEGER) {
			cerr << "Operadores aritmeticos aplicados a expresses no inteiras\n";
			exit(1);
		}
	}
	
		
	return v; 
}

