#ifndef UNARY_EXPRESSION_H
#define UNARY_EXPRESSION_H

#include <string>
#include "ast.h"
#include "expression.h"

using namespace std;

class UnaryExpression : public Expression {
	private:
		Expression *expression;
		string op;

	public:
		UnaryExpression(SrcPosition *position, Expression *expression, string op)
				: Expression(position), Ast(position), expression(expression), op(op) {}

		UnaryExpression(const UnaryExpression& unaryExpression) : Expression(unaryExpression.position), Ast(position) {
			*(this) = unaryExpression;
		}

		virtual ~UnaryExpression() {
			delete this->expression;
		}

		Expression *getExpression() {
			return this->expression;
		}

		string getOp() const {
			return this->op;
		}

		virtual Ast& operator = (const Ast& ast);
		virtual bool operator == (const Ast& ast) const;

		virtual VisitorReturn *visit(Visitor& visitor) {
			return visitor.visitUnaryExpression(this);
		}

		virtual Ast& clone() const {
			return *(new UnaryExpression(new SrcPosition(*(this->position)),
					 this->expression ? dynamic_cast<Expression*>(&(this->expression->clone())) : 0,
					 this->op));
		}
};

#endif
