#ifndef VISITOR_H
#define VISITOR_H

#include "visitorreturn.h"

class Action;
class ActStatement;
class ArrayReference;
class ArrayType;
class AssignStatement;
class Ast;
class Behavior;
class BehaviorStatement;
class BinaryExpression;
class Declaration;
class DoStatement;
class Expression;
class ExpressionList;
class FieldDeclaration;
class FieldReference;
class FieldType;
class Guard;
class GuardedStatement;
class GuardExpression;
class GuardReceive;
class GuardTimeout;
class Identifier;
class IdentifierList;
class IfStatement;
class Import;
class IntegerLiteral;
class LeftSide;
class LeftSideList;
class Message;
class MultipleArrayReference;
class Process;
class Program;
class RangeType;
class ReceiveStatement;
class SendStatement;
class SequentialAction;
class SequentialBehavior;
class SequentialDeclaration;
class SequentialExpressionList;
class SequentialFieldDeclaration;
class SequentialGuardedStatement;
class SequentialIdentifierList;
class SequentialImport;
class SequentialLeftSideList;
class SequentialMessage;
class SequentialProcess;
class SequentialStatement;
class SimpleAction;
class SimpleArrayReference;
class SimpleBehavior;
class SimpleDeclaration;
class SimpleExpressionList;
class SimpleFieldDeclaration;
class SimpleGuardedStatement;
class SimpleIdentifierList;
class SimpleImport;
class SimpleLeftSideList;
class SimpleMessage;
class SimpleProcess;
class SimpleStatement;
class SkipStatement;
class StartStatement;
class Statement;
class StopStatement;
class StringLiteral;
class Type;
class UnaryExpression;
class Visitor {
	public:
		virtual ~Visitor() {}
		virtual VisitorReturn *visitAction(Action *action) = 0;
		virtual VisitorReturn *visitActStatement(ActStatement *actStatement) = 0;
		virtual VisitorReturn *visitArrayReference(ArrayReference *arrayReference) = 0;
		virtual VisitorReturn *visitArrayType(ArrayType *arrayType) = 0;
		virtual VisitorReturn *visitAssignStatement(AssignStatement *assignStatement) = 0;
		virtual VisitorReturn *visitAst(Ast *ast) = 0;
		virtual VisitorReturn *visitBehavior(Behavior *behavior) = 0;
		virtual VisitorReturn *visitBehaviorStatement(BehaviorStatement *behaviorStatement) = 0;
		virtual VisitorReturn *visitBinaryExpression(BinaryExpression *binaryExpression) = 0;
		virtual VisitorReturn *visitDeclaration(Declaration *declaration) = 0;
		virtual VisitorReturn *visitDoStatement(DoStatement *doStatement) = 0;
		virtual VisitorReturn *visitExpression(Expression *expression) = 0;
		virtual VisitorReturn *visitExpressionList(ExpressionList *expressionList) = 0;
		virtual VisitorReturn *visitFieldDeclaration(FieldDeclaration *fieldDeclaration) = 0;
		virtual VisitorReturn *visitFieldReference(FieldReference *fieldReference) = 0;
		virtual VisitorReturn *visitFieldType(FieldType *fieldType) = 0;
		virtual VisitorReturn *visitGuard(Guard *guard) = 0;
		virtual VisitorReturn *visitGuardedStatement(GuardedStatement *guardedStatement) = 0;
		virtual VisitorReturn *visitGuardExpression(GuardExpression *guardExpression) = 0;
		virtual VisitorReturn *visitGuardReceive(GuardReceive *guardReceive) = 0;
		virtual VisitorReturn *visitGuardTimeout(GuardTimeout *guardTimeout) = 0;
		virtual VisitorReturn *visitIdentifier(Identifier *identifier) = 0;
		virtual VisitorReturn *visitIdentifierList(IdentifierList *identifierList) = 0;
		virtual VisitorReturn *visitIfStatement(IfStatement *ifStatement) = 0;
		virtual VisitorReturn *visitImport(Import *import) = 0;
		virtual VisitorReturn *visitIntegerLiteral(IntegerLiteral *integerLiteral) = 0;
		virtual VisitorReturn *visitLeftSide(LeftSide *leftSide) = 0;
		virtual VisitorReturn *visitLeftSideList(LeftSideList *leftSideList) = 0;
		virtual VisitorReturn *visitMessage(Message *message) = 0;
		virtual VisitorReturn *visitMultipleArrayReference(MultipleArrayReference *multipleArrayReference) = 0;
		virtual VisitorReturn *visitProcess(Process *process) = 0;
		virtual VisitorReturn *visitProgram(Program *program) = 0;
		virtual VisitorReturn *visitRangeType(RangeType *rangeType) = 0;
		virtual VisitorReturn *visitReceiveStatement(ReceiveStatement *receiveStatement) = 0;
		virtual VisitorReturn *visitSendStatement(SendStatement *sendStatement) = 0;
		virtual VisitorReturn *visitSequentialAction(SequentialAction *sequentialAction) = 0;
		virtual VisitorReturn *visitSequentialBehavior(SequentialBehavior *sequentialBehavior) = 0;
		virtual VisitorReturn *visitSequentialDeclaration(SequentialDeclaration *sequentialDeclaration) = 0;
		virtual VisitorReturn *visitSequentialExpressionList(SequentialExpressionList *sequentialExpressionList) = 0;
		virtual VisitorReturn *visitSequentialFieldDeclaration(SequentialFieldDeclaration *sequentialFieldDeclaration) = 0;
		virtual VisitorReturn *visitSequentialGuardedStatement(SequentialGuardedStatement *sequentialGuardedStatement) = 0;
		virtual VisitorReturn *visitSequentialIdentifierList(SequentialIdentifierList *sequentialIdentifierList) = 0;
		virtual VisitorReturn *visitSequentialImport(SequentialImport *sequentialImport) = 0;
		virtual VisitorReturn *visitSequentialLeftSideList(SequentialLeftSideList *sequentialLeftSideList) = 0;
		virtual VisitorReturn *visitSequentialMessage(SequentialMessage *sequentialMessage) = 0;
		virtual VisitorReturn *visitSequentialProcess(SequentialProcess *sequentialProcess) = 0;
		virtual VisitorReturn *visitSequentialStatement(SequentialStatement *sequentialStatement) = 0;
		virtual VisitorReturn *visitSimpleAction(SimpleAction *simpleAction) = 0;
		virtual VisitorReturn *visitSimpleArrayReference(SimpleArrayReference *simpleArrayReference) = 0;
		virtual VisitorReturn *visitSimpleBehavior(SimpleBehavior *simpleBehavior) = 0;
		virtual VisitorReturn *visitSimpleDeclaration(SimpleDeclaration *simpleDeclaration) = 0;
		virtual VisitorReturn *visitSimpleExpressionList(SimpleExpressionList *simpleExpressionList) = 0;
		virtual VisitorReturn *visitSimpleFieldDeclaration(SimpleFieldDeclaration *simpleFieldDeclaration) = 0;
		virtual VisitorReturn *visitSimpleGuardedStatement(SimpleGuardedStatement *simpleGuardedStatement) = 0;
		virtual VisitorReturn *visitSimpleIdentifierList(SimpleIdentifierList *simpleIdentifierList) = 0;
		virtual VisitorReturn *visitSimpleImport(SimpleImport *simpleImport) = 0;
		virtual VisitorReturn *visitSimpleLeftSideList(SimpleLeftSideList *simpleLeftSideList) = 0;
		virtual VisitorReturn *visitSimpleMessage(SimpleMessage *simpleMessage) = 0;
		virtual VisitorReturn *visitSimpleProcess(SimpleProcess *simpleProcess) = 0;
		virtual VisitorReturn *visitSimpleStatement(SimpleStatement *simpleStatement) = 0;
		virtual VisitorReturn *visitSkipStatement(SkipStatement *skipStatement) = 0;
		virtual VisitorReturn *visitStartStatement(StartStatement *startStatement) = 0;
		virtual VisitorReturn *visitStatement(Statement *statement) = 0;
		virtual VisitorReturn *visitStopStatement(StopStatement *stopStatement) = 0;
		virtual VisitorReturn *visitStringLiteral(StringLiteral *stringLiteral) = 0;
		virtual VisitorReturn *visitType(Type *type) = 0;
		virtual VisitorReturn *visitUnaryExpression(UnaryExpression *unaryExpression) = 0;
};

#endif
