%{
/**
 * Protomake Lexer by Fbio Guerra, Igor Canana & Thiago Souto Maior
 *
 * This is a lex file that is used with flex to create a lexer for our protocol
 * compiler.
 */	
#include <map>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
using namespace std;

#include "nodes/protomake.h"
enum EtherType {AUTHOR, EMAIL, NAME, PORT, TRANSPORT};
struct EtherInfoType
{
    EtherType type;
    char *string;
};

/** Includes token definitions from the Parser */
#include "protomakeParse.hpp"

/** Global variable that counts the number of lines read */
int lines;

/** 
 * Function that reformats a multi-string so that it appears to be a single-line
 * string.
 */
static char *formatMultiString(char *);
%}

id		[a-zA-Z][a-zA-Z0-9]*

string	\"[^\"\n]*\"
pString	[^\"\n]*[ \t]*
mString	\"({pString}\\\n)+{pString}\"
number	[0-9]+
white	[ \t]

%%

!@Author   { return tEI_AUTHOR; } 
!@Email    { return tEI_EMAIL; }
!@Name     { return tEI_NAME; }
!@Acrynom   { return tEI_ABBREV; }
!@Port     { return tEI_PORT; }
!@Transport    { return tEI_TRANS; }
!\{.*\}!\n	{ lines++; return tETHERDESC; }
#.*\n		{ /** Coments */ lines++; }
external    { return tEXTERNAL; }
message 	{ return tMESSAGE; }
import	 	{ return tIMPORT; }
begin		{ return tBEGIN; }
end			{ return tEND; }
bits		{ return tBITS; }
bytes		{ return tBYTES; }
process		{ return tPROCESS; }
const		{ return tCONST; }
var			{ return tVAR; }
integer		{ return tINTEGER; }
address		{ return tADDRESS; }
client_address { return tCLIENT_ADDRESS; }
server_address { return tSERVER_ADDRESS; }
array		{ return tARRAY; }
of			{ return tOF; }
rcv			{ return tRCV; }
tcp			{ return tTCP; }
udp			{ return tUDP; }
from		{ return tFROM; }
timeout		{ return tTIMEOUT; }
skip		{ return tSKIP; }
send		{ return tSEND; }
to			{ return tTO; }
if			{ return tIF; }
fi			{ return tFI; }
do			{ return tDO; }
od			{ return tOD; }
act			{ return tACT; }
in			{ return tIN; }
start		{ return tSTART; }
stop		{ return tSTOP; }
behavior { return tBEHAVIOR; }

"+"			{ yylval.string = strdup(yytext); return tPLUS; }
"-"			{ yylval.string = strdup(yytext); return tMINUS; }
"*"			{ yylval.string = strdup(yytext); return tTIMES; }
"/"			{ yylval.string = strdup(yytext); return tDIVIDE; }
"&"			{ yylval.string = strdup(yytext); return tBAND; }
"|"			{ yylval.string = strdup(yytext); return tBOR; }
"~"			{ yylval.string = strdup(yytext); return tNEG; }
"<>"		{ yylval.string = strdup(yytext); return tDIFFERENT; }
"<="		{ yylval.string = strdup(yytext); return tLTE; }
">="		{ yylval.string = strdup(yytext); return tGTE; }
"="			{ yylval.string = strdup(yytext); return tEQUAL; }
"<"			{ yylval.string = strdup(yytext); return tLT; }
">"			{ yylval.string = strdup(yytext); return tGT; }
"[]"		{ return tGUARD; }
"["			{ return tLBRACKET; }
"]"			{ return tRBRACKET; }
"("			{ return tLPAR; }
")"			{ return tRPAR; }
","			{ return tCOMMA; }
":"			{ return tCOLON; }
";"			{ return tSCOLON; }
"->"		{ return tACTION; }
"."			{ return tDOT; }
".."		{ return tRANGE; }
":="		{ return tASSIGN; }

{number}	{ yylval.number = atoi(yytext); return tNUMBER; }
{id}		{ yylval.string = strdup(yytext); return tID; }
{string}	{ yylval.string = strdup(yytext); return tSTRING; }
{mString}	{ yylval.string = formatMultiString(yytext); return tSTRING; }


{white}+	{ /* ignore */ }
\n			{ /** line counter */ lines++; }
.			{ printf("Erro na linha %d: %s!\n", lines + 1, yytext); exit(0);}

%%

/** Processes the multi-string and turns it into a single-line string */
static char *formatMultiString(char *s)
{
	char *str = (char *) malloc(strlen(s) + 1);	
	char *token;

	str[0] = '\0';
	token = strtok(s, "\\\n");
	do
	{
		strcat(str, token);
	} while ((token = strtok(NULL, "\\\n")));


	return str;
}
