%{
/**
 * Protomake Parser by Fbio Guerra, Igor Canana & Thiago Souto Maior
 *
 * This is a yacc file that is used with bison to create a parser for our
 * protocol compiler.
 */

#define C		0x1
#define JAVA	0x2

#define VERSAO "O.1.1"

#include <map>
#include <vector>
#include <typeinfo>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
using namespace std;

#include "nodes/protomake.h"

/** Forward defenition of yylex so the c++ compiler won't complain! :)*/
int yylex(void);

/**
 * Global variable, defined in th lex file, used to count the number of lines
 * read.
 */
extern int lines;

/**
 * Global varible that will hold the abstract syntax tree.
 */
Program *root;

enum EtherType {AUTHOR, EMAIL, NAME, ABBREV, PORT, TRANSPORT};
struct EtherInfoType
{
    EtherType type;
    char *string;
    int lines;
};
multimap<EtherType, EtherInfoType> *etherInfo;

struct FieldInfo
{
    char *identifier;
    Expression *size;
    FieldType::FType bitByte;
    Expression *init;
    char *etherDesc;
   
};
map<string, vector<FieldInfo> > fiMap;
vector<FieldInfo> fiVec;

int yyerror(char *msg);

/** Generates a plugin for ethereal */
static void generateEtherealPlugin(char *baseDir,
								   map<string, vector<FieldInfo> > *fiMap,
                                   multimap<EtherType, EtherInfoType> *eit);

static void writeHeader(FILE *plugin, char *fileName,
                        multimap<EtherType, EtherInfoType> *eit); 

static void writeDissector(FILE *plugin, map<string, vector<FieldInfo> > *fiMap,
                           multimap<EtherType, EtherInfoType> *eit);

static void writeRest(FILE *plugin, char *trans, char *port, char *abbrev);

%}

/** The different types that tokens (terminals) and non-terminals can assume */
%union {
	long number;
	char *string;
	SimpleImport *simpImp;
	Import *import;
	IdentifierList *idLst;
	Behavior *behavior;
	SimpleBehavior *simpBhv;
	Message *message;
	SimpleMessage *simpMsg;
	FieldType::FType fType;
	FieldType *fieldTyp;
	Expression *exp;
	FieldDeclaration *fieldDcl;
	SimpleFieldDeclaration *simpFld;
	StringLiteral *strLit;
	Process *proc;
	SimpleProcess *simpProc;
	Declaration *decl;
	SimpleDeclaration *simpDecl;	
	Action *action;
	SimpleAction *simpAct;
	Type *type;
	Guard *guard;
	Statement *stmt;
	SimpleStatement *simpStmt;
	ArrayReference *arrayRef;
	LeftSideList *leftSideLst;
	LeftSide *leftSide;
	GuardedStatement *guardStmt;
	SimpleGuardedStatement *simpGuardStmt;
	FieldReference *fieldRef;
	ExpressionList *expLst;
    EtherInfoType eiType;
    multimap<EtherType, EtherInfoType> *strList;
}

/**
 * Definition of every token used by the parser. Theses definitions are used by
 * the lexer to notify the parser which token it picked up.
 */
%token<string>  tEI_NAME
%token<string>  tEI_ABBREV
%token<string>  tEI_PORT
%token<string>  tEI_TRANS
%token<string>  tEI_AUTHOR
%token<string>  tEI_EMAIL
%token<string>	tETHERDESC
%token<string>	tBEHAVIOR
%token<string> 	tID
%token<string>	tSTRING
%token<number>	tNUMBER
%token<string>	tMESSAGE
%token<string>	tIMPORT
%token<string>	tEXTERNAL
%token<string>	tBEGIN
%token<string>	tEND
%token<string>	tBITS
%token<string>	tBYTES
%token<string>	tPROCESS
%token<string>	tCONST
%token<string>	tVAR
%token<string>	tINTEGER
%token<string>	tADDRESS
%token<string>	tCLIENT_ADDRESS
%token<string>	tSERVER_ADDRESS
%token<string>	tARRAY
%token<string>	tOF
%token<string>	tRCV
%token<string>	tFROM
%token<string>	tTIMEOUT
%token<string>	tSKIP
%token<string>	tSEND
%token<string>	tTO
%token<string>	tIF
%token<string>	tFI
%token<string>	tDO
%token<string>	tOD
%token<string>	tACT
%token<string>	tIN
%token<string>	tTHROUGH
%token<string>	tTCP
%token<string>	tUDP
%token<string>	tPLUS
%token<string>	tMINUS
%token<string>	tTIMES
%token<string>	tDIVIDE
%token<string>	tBAND
%token<string>	tBOR
%token<string>	tNEG
%token<string>	tDIFFERENT
%token<string>	tLTE
%token<string>	tGTE
%token<string>	tEQUAL
%token<string>	tLT
%token<string>	tGT
%token<number>	tGUARD
%token<number>	tLBRACKET
%token<number>	tRBRACKET
%token<number>	tLPAR
%token<number>	tRPAR
%token<number>	tCOMMA
%token<number>	tCOLON
%token<number>	tSCOLON
%token<number>	tACTION
%token<number>	tDOT
%token<number>	tRANGE
%token<number>	tASSIGN
%token<number>  tDQUOTE
%token<string>	tSTART
%token<string>	tSTOP

/** Type Definition of every non-terminal used by the parser. */
%type<string>	Program
%type<strList>	EtherInfo
%type<strList>   RealEtherInfo
%type<eiType>   EtherOptions
%type<import>	Imports
%type<import>	ReallyImport
%type<simpImp>	Import
%type<behavior>	Behaviors
%type<behavior>	RealBehavior
%type<simpBhv>	Behavior
%type<message>	Messages
%type<proc>		Processes
%type<simpProc> RealProcess
%type<simpMsg>	Message
%type<number>	External
%type<fieldDcl>	MessageBody
%type<fieldDcl>	Fields
%type<simpFld>	Field
%type<exp>		Init
%type<strLit>	EtherDescription
%type<fieldTyp>	FieldType
%type<fType>	BitByte
%type<simpDecl>	DeclarationTypes
%type<decl>		Declarations
%type<idLst>	IDs
%type<type>		Type
%type<action>	Actions
%type<simpAct>	Action
%type<guard>	GuardedAction
%type<exp>		Expression
%type<fieldRef>		FieldReference
%type<arrayRef>		ArrayReference
%type<exp>		ArrayPosition
%type<stmt>		Statements
%type<simpStmt>	Statement
%type<leftSideLst>	LeftSides
%type<leftSide>	LeftSide
%type<expLst>	Expressions
%type<guardStmt>	GuardedStatements
%type<simpGuardStmt>	GuardedStatement

/** Precedence and associativity conflict resolvers*/
%nonassoc tBOR, tBAND
%nonassoc tNEG
%nonassoc tLT, tLTE, tGT, tGTE, tEQUAL, tDIFFERENT
%left tASSIGN
%left tPLUS, tMINUS
%left tTIMES, tDIVIDE
%left UMINUS

%%

/** Starting point */
/** Program Terminado*/
Program	: EtherInfo Imports Messages Behaviors Processes
			{
				root = new Program(new SrcPosition(1),  $2, $4, $3, $5);
    		} 
		;
			
/** Terminado!!! */
EtherInfo	: RealEtherInfo { etherInfo = $1;}
            | { etherInfo = NULL; }
			;

/** Terminado!!! */
RealEtherInfo   : EtherOptions
                    {
                        $$ = new multimap<EtherType, EtherInfoType>;
                        $$->insert(make_pair($1.type, $1));
                    }
   
                | RealEtherInfo EtherOptions
                    {
                        $1->insert(make_pair($2.type, $2));
                        $$ = $1;
                    }
                ;

/** Terminado!!! */
EtherOptions    : tEI_AUTHOR tSTRING
                    {
                        EtherInfoType eiType;

                        eiType.type = AUTHOR;
                        eiType.string = $2;
                        eiType.lines = lines;
                        $$ = eiType;
                    }

                | tEI_EMAIL tSTRING
                    {
                        EtherInfoType eiType;

                        eiType.type = EMAIL;
                        eiType.string = $2;
                        eiType.lines = lines;
                        $$ = eiType;
                    }

                | tEI_NAME tSTRING
                    {
                        EtherInfoType eiType;

                        eiType.type = NAME;
                        eiType.string = $2;
                        eiType.lines = lines;
                        $$ = eiType;
                    }

                | tEI_ABBREV tSTRING
                    {
                        EtherInfoType eiType;

                        eiType.type = ABBREV;
                        eiType.string = $2;
                        eiType.lines = lines;
                        $$ = eiType;
                    }

                | tEI_PORT tNUMBER
                    {
                        EtherInfoType eiType;

                        eiType.type = PORT;
                        eiType.string = new char[15];
                        sprintf(eiType.string, "%ld", $2);
                        eiType.lines = lines;
                        $$ = eiType;
                    }

                | tEI_TRANS tUDP
                    {
                        EtherInfoType eiType;

                        eiType.type = TRANSPORT;
                        eiType.string = "udp";
                        eiType.lines = lines;
                        $$ = eiType;
                    }

                | tEI_TRANS tTCP
                    {
                        EtherInfoType eiType;

                        eiType.type = TRANSPORT;
                        eiType.string = "tcp";
                        eiType.lines = lines;
                        $$ = eiType;
                    }
                ;

/** Protomake imports */
/** Imports terminado!!! */
Imports	: ReallyImport { $$ = $1; }
		| { $$ = NULL; }
		;

/** ReallyImport terminado!!! */
ReallyImport	: Import
					{
						/** O ltimo SequentialImport s tem um SimpleImport*/
						$$ = $1;
					}
				| ReallyImport Import
					{
						$$ = new SequentialImport($2->getPosition(), $1, $2);
					}
				;

/** Import terminado!!! */
Import	: tIMPORT tSTRING
			{
				/** Gera um SimpleImport */
				SrcPosition *srcPos = new SrcPosition(lines);
				StringLiteral *str = new StringLiteral(srcPos, $2);
				$$ = new SimpleImport(srcPos,  str);
			}
		| error { yyerror("Import sintax error!"); }
		;

/** Behaviors */
/** Behaviors terminado!!! */
Behaviors	: RealBehavior { $$ = $1; }
			| { $$ = NULL}
			;

/** RealBehavior terminado!!! */
RealBehavior	: Behavior
					{
						$$ = $1; 
					}
				| RealBehavior Behavior
					{
						$$ = new SequentialBehavior($2->getPosition(), $1, $2);
					}
				;

/** Behavior terminado!!! */
Behavior	: tBEHAVIOR IDs
				{
					$$ = new SimpleBehavior($2->getPosition(), $2);
				}
			;

/** Messages */
/** Messages terminado!!! */
Messages	: Message { $$ = $1; }
			| Messages Message { $$ = new SequentialMessage($2->getPosition(), $1, $2); }
			;

/** Message definition */
/** Message terminado */
Message	: External tMESSAGE tID tLPAR tID tCOMMA tID tRPAR MessageBody
			{
				$$ = new SimpleMessage($9->getPosition(), new Identifier($9->getPosition(), $3),
									   $9, new Identifier($9->getPosition(), $5),
									   new Identifier($9->getPosition(), $7), 0);

                /** For the potential plugin */
                fiMap[$3] = fiVec;
                fiVec.clear();
			}
		| External tMESSAGE tID MessageBody
			{
				$$ = new SimpleMessage($4->getPosition(), new Identifier($4->getPosition(), $3),
									   $4, NULL, NULL, 0);

                /** For the potential plugin */
                fiMap[$3] = fiVec;
                fiVec.clear();
            }
		;

/** Optional "external" qualifier */
/** External terminado!!! */
External	: tEXTERNAL { $$ = 1; }
			| { $$ = 0 }
			;

/** Message body definition */
/** MessageBody terminado!!! */
MessageBody	: tBEGIN Fields tEND { $$ = $2; }
			| error { yyerror("Message body sintax error!"); }
			;

/** Fields */
/** Fields terminado!!! */
Fields	: Field { $$ = $1; }
		| Fields tCOMMA Field { $$ = new SequentialFieldDeclaration($3->getPosition(), $1, $3); }
		;

/** Field definition */
/** Field terminado!!! */
Field	: tID tCOLON FieldType Init EtherDescription
			{
				Identifier *id = new Identifier($3->getPosition(), $1);
				$$ = new SimpleFieldDeclaration($3->getPosition(), id, $3, $4, $5);

                /** For the potential plugin... */
                FieldInfo fi;

                fi.identifier = $1;
                fi.size = $3->getExpression();
                fi.bitByte = $3->getType();
                fi.init = $4;
                if ($5)
                {
                    fi.etherDesc = new char[strlen($5->getStr().c_str()) + 1];
                    strcpy(fi.etherDesc, $5->getStr().c_str());
                } else fi.etherDesc = NULL;
                
                fiVec.push_back(fi);
			}
		| error { yyerror("Field declaration sintax error!"); }
		;

/** EtherDescription terminado */
EtherDescription	: tETHERDESC { $$ = new StringLiteral(new SrcPosition(lines), $1); }
					| { $$ = NULL}
					;

/** Optional field initializer */
/** Init terminado!!! */
Init	: tEQUAL Expression { $$ = $2; }
		| { $$ = NULL; }
		;

/** Field type definition */
/** FieldType terminado!!! */
FieldType	: Expression BitByte
				{
					$$ = new FieldType($1->getPosition(), $1, $2);
				}
			| error { yyerror("Field type sintaxe error!"); }
			;

/** Type of field */
/** BitByte terminado!!! */
BitByte	: tBITS { $$ = FieldType::BIT; }
		| tBYTES { $$ = FieldType::BYTE; }
		| error { yyerror("Undefined field type!"); }
		;

/** Process definition. Only one process is alowed! May change... */
/** Terminado! */
Processes	: RealProcess
			  {
			  		$$ = $1;
			  }
			| Processes RealProcess
			  {
			  		$$ = new SequentialProcess($2->getPosition(), $1, $2);
			  }
			;

/** Terminado...*/
RealProcess	: tPROCESS tID Declarations tBEGIN Actions tEND
			    {
					$$ = new SimpleProcess($3->getPosition(), 
						     new Identifier($3->getPosition(), $2), $3, $5, 0);
				}
			| error { yyerror("Process defenition sintax error!"); }
			;

/** Constant definition */
/** Terminou */
DeclarationTypes	: tCONST IDs tCOLON Type Init 
						{ 
							$$ = new SimpleDeclaration(new SrcPosition(lines), $2, $4, $5, true); 
						}
					| tVAR IDs tCOLON Type Init 
						{
							$$ = new SimpleDeclaration(new SrcPosition(lines), $2, $4, $5, false);
						} 
						
					| { $$ = NULL; }
					| error { yyerror("DeclarationType syntax error!"); }
					;

/** Declarations */
/** Terminou*/
Declarations	: DeclarationTypes 
					{
						$$ = $1;
					}
				| Declarations tSCOLON DeclarationTypes 
					{ 
						$$ = new SequentialDeclaration($3->getPosition(), $1, $3);
					}
				;


/** More than one identifier can be declared in a single line*/
/** IDs terminado!!! */
IDs	: tID
		{
			Identifier *identifier = new Identifier(new SrcPosition(lines), $1);
			$$ = new SimpleIdentifierList(identifier->getPosition(), identifier);
		}
		
	| IDs tCOMMA tID
		{
			Identifier *identifier = new Identifier(new SrcPosition(lines), $3);
			SimpleIdentifierList *simpIdList = new SimpleIdentifierList(identifier->getPosition(), identifier);
			$$ = new SequentialIdentifierList(identifier->getPosition(), $1, simpIdList);					 
		}
		
	| error { yyerror("Identifier list sintax error!"); }
	;

/** Type definition */
/** Terminou*/
Type	: tINTEGER {$$ = new Type(new SrcPosition(lines), Type::INTEGER);}
		| tNUMBER tRANGE tNUMBER { $$ = new RangeType(new SrcPosition(lines), Type::RANGE, $1, $3); }
		| tADDRESS {$$ = new Type(new SrcPosition(lines), Type::ADDRESS);}
		| tCLIENT_ADDRESS {$$ = new Type(new SrcPosition(lines), Type::CLIENT_ADDRESS);}
		| tSERVER_ADDRESS {$$ = new Type(new SrcPosition(lines), Type::SERVER_ADDRESS);}
		| tARRAY tLBRACKET tNUMBER tRBRACKET tOF Type {$$ = new ArrayType(new SrcPosition(lines), Type::ARRAY, $6, $3);}
		| error {yyerror("Undefined type!");}
		;

/** Actions */
/** Terminou*/
Actions	: Action {$$ = $1;}
		| Actions tGUARD Action {$$ = new SequentialAction($3->getPosition(), $1, $3);}
		;

/** 
 * An action is a sequence of statements that will be executed if the
 * GuardedAction is satisfied
 */
 /** Terminou*/
Action	: GuardedAction tACTION Statements {$$ = new SimpleAction($1->getPosition(), $1, $3);}
	    | error {yyerror("Action syntax error!");}
		;

/** GuardedAction definition */
/** GuardedAction timeout!!! */
GuardedAction	: Expression { $$ = new GuardExpression($1->getPosition(), $1); }
				| tRCV tID tFROM tID
					{
						SrcPosition *pos = new SrcPosition(lines);
						
						ReceiveStatement *rs = new ReceiveStatement(pos, new Identifier(pos, $2),
																	new Identifier(pos, $4), 0);
						$$ = new GuardReceive(pos, rs);
					}
				| tTIMEOUT tID
					{
						SrcPosition *pos = new SrcPosition(lines);
					
						$$ = new GuardTimeout(pos, new Identifier(pos, $2));
					}
				| error { yyerror("Invalid guard!"); }
				;

/** Expression definition */
/** Terminou*/
Expression	: tLPAR Expression tRPAR { $$ = $2; }
		   
			| Expression tEQUAL Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tGT Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tLT Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tGTE Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tLTE Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tDIFFERENT Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tBOR Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tBAND Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tPLUS Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tMINUS Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tTIMES Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| Expression tDIVIDE Expression
				{ $$ = new BinaryExpression(new SrcPosition(lines), $1, $3, $2); }
				
			| tNEG Expression { $$ = new UnaryExpression(new SrcPosition(lines), $2, $1); }
			
			| tMINUS Expression %prec UMINUS { $$ = new UnaryExpression(new SrcPosition(lines), $2, $1); }

			| FieldReference { $$ = $1; }
			| ArrayReference { $$ = $1; }
			| tID { $$ = new Identifier(new SrcPosition(lines), $1); }
			| tNUMBER { $$ = new IntegerLiteral(new SrcPosition(lines), $1); }
			| tSTRING { $$ = new StringLiteral(new SrcPosition(lines), $1); }
			;

/** Fieldreferencing */
/**Terminou*/
FieldReference	: tID tDOT tID 
			   		{ 
			   			SrcPosition *pos = new SrcPosition(lines);
						$$ = new FieldReference(pos, new Identifier(pos, $1), new Identifier(pos, $3)); 
					}
				;

/** Accessing array postions */
/**Terminou*/
ArrayReference	: tID ArrayPosition  { $$ = new SimpleArrayReference($2->getPosition(), new Identifier($2->getPosition(), $1), $2); }
				| ArrayReference ArrayPosition {$$ = new MultipleArrayReference($1->getPosition(), $1, $2);}
				;

/** How to specify an array position*/
/**Terminou*/
ArrayPosition	: tLBRACKET Expression tRBRACKET { $$ = $2; }
				;

/** Statements */
/**Terminou*/
Statements	: Statement {$$ = $1;}
			| Statements tSCOLON Statement {$$ = new SequentialStatement($3->getPosition(), $1, $3);}
			;

/** Statement definitions */
/** Statement terminado!!! */
Statement	: tSKIP {$$ = new SkipStatement(new SrcPosition(lines));}
			| LeftSides tASSIGN Expressions {$$ = new AssignStatement($1->getPosition(), $1, $3);}
			| tRCV tID tFROM tID
				{
					SrcPosition *pos = new SrcPosition(lines);
					$$ = new ReceiveStatement(pos, new Identifier(pos, $2), new Identifier(pos, $4), 0);
				}
			| tSEND tID tTO tID
				{
					SrcPosition *pos = new SrcPosition(lines);
					$$ = new SendStatement(pos, new Identifier(pos, $2), new Identifier(pos, $4), 0);
				}
			| tIF GuardedStatements tFI { $$ = new IfStatement($2->getPosition(), $2); }
			| tDO GuardedStatement tOD { $$ = new DoStatement($2->getPosition(), $2); }
			| tACT tID tIN Expression { $$ = new ActStatement($4->getPosition(), new Identifier($4->getPosition(), $2) ,$4); }
			| tSTART tID 
				{
					SrcPosition *pos = new SrcPosition(lines);
					$$ = new StartStatement(pos, new Identifier(pos, $2));
				}
			| tSTOP { $$ = new StopStatement(new SrcPosition(lines)); }

			| tID tDOT tID tLPAR Expressions tRPAR
				{
					$$ = new BehaviorStatement($5->getPosition(),
											   new Identifier($5->getPosition(), $1),
											   new Identifier($5->getPosition(), $3),
											   $5);
				}
			;

/** LeftSides */
/** LeftSides terminado!!! */
LeftSides	: LeftSide { $$ = new SimpleLeftSideList($1->getPosition(), $1); }
			| LeftSides tCOMMA LeftSide
				{
					$$ = new SequentialLeftSideList($3->getPosition(), $1,
													new SimpleLeftSideList($3->getPosition(), $3));
				}
			;

/** Who can be on the left side? */
/** LeftSide terminado!!! */
LeftSide	: tID { $$ = new Identifier(new SrcPosition(lines), $1); }
			| FieldReference { $$ = $1; }
			| ArrayReference { $$ = $1; }
			;

/** Expressions */
/** Expressions terminado!!! */
Expressions	: Expression { $$ = new SimpleExpressionList($1->getPosition(), $1); }
			| Expressions tCOMMA Expression
				{
					$$ = new SequentialExpressionList($1->getPosition(), $1,
													  new SimpleExpressionList($1->getPosition(), $3));
				}
			;

/** GuardedStatements */
/** GuardedStatements terminado!!! */
GuardedStatements	: GuardedStatement { $$ = $1; }
					| GuardedStatements tGUARD GuardedStatement
						{
							$$ = new SequentialGuardedStatement($1->getPosition(), $1, $3);
						}
					;

/** GuardedStatement definition */
/** GuardedStatement terminado!!! */
GuardedStatement	: Expression tACTION Statements
				 		{
							$$ = new SimpleGuardedStatement($1->getPosition(), $1, $3);
						}
					;

%%

int main(int argc, char **argv)
{
	extern FILE *yyin;
	int opt;
	int generator = 0;
	char *baseDir = strdup("./");
	char *inputFile = "";
	char *baseName;

	opterr = 0; // supress error message from getopt
	do {
		opt = getopt(argc, argv, "+:hcjd:i:");
		switch (opt) {
			case 'c':
				generator |= C;
				break;
			
			case 'j':
				generator |= JAVA;
				break;

			case 'h':
				fprintf(stderr, "\nProtomake verso %s por Igor Canana, Thiago Souto Maior, Fbio Guerra e\n", VERSAO);
				fprintf(stderr, "Rodrigo Cavalcanti. Copyright 2004\n\n");
				fprintf(stderr, "Uso: protomake [-j] [-c] [-h] -i file [-d out]\n\n");
				fprintf(stderr, "Opes:\n");
				fprintf(stderr, "\t-j\t\tGera cdigo em Java.\n");
				fprintf(stderr, "\t-c\t\tGera cdigo em C (default).\n");
				fprintf(stderr, "\t-h\t\tMostra o help.\n");
				fprintf(stderr, "\t-i file\t\tL a descrio do protocolo a partir do arquivo file.\n");
				fprintf(stderr, "\t-d out\t\tGera a saida na pasta out (default = pasta atual).\n\n");
				fprintf(stderr, "As opes de gerao de cdigo podem ser usadas simultaneamente(-j -c).\n");
				fprintf(stderr, "A ordem dos parmetros  irrelevante.\n\n");
				exit(1);
				break;

			case 'd':
				free(baseDir);
				baseDir = strdup(optarg);

				if (mkdir(baseDir, 00755) == -1)
					if (errno != EEXIST) {
						fprintf(stderr, "No foi possvel criar o diretrio %s!\n", baseDir);
						exit(1);
					}

				{	
					char *plugin = new char[strlen(baseDir) + 15];
		
					sprintf(plugin, "%s/plugin", baseDir);
					if (mkdir(plugin, 00755) == -1)
						if (errno != EEXIST) {
							fprintf(stderr, "No foi possvel crirar o diretrio %s!\n", plugin);
							exit(1);
						}
	
					delete[] plugin;
				}
				break;

			case 'i':
				int i;
				int dot;
				
				inputFile = strdup(optarg);
				for (i = strlen(inputFile); i >= 0 && inputFile[i] != '/'; i--)
					if (inputFile[i] == '.')
						dot = i;

				baseName = strdup(&inputFile[i+1]);
				baseName[dot] = 0;
				break;

			case '?':
				printf("-%c no  uma opo vlida!\n", optopt);
				exit(1);
				break;

			case ':':
				printf("Falta um argumento para -%c!\n", optopt);
				exit(1);
				break;
				
			default:
				break;
		}
	} while (opt != -1);

	if (!generator)
		generator = C;

    if (!strcmp(baseDir, "./")) {
		char *plugin = new char[strlen(baseDir) + 15];
		
		sprintf(plugin, "%s/plugin", baseDir);
		if (mkdir(plugin, 00755) == -1)
			if (errno != EEXIST) {
				fprintf(stderr, "No foi possvel crirar o diretrio %s!\n", plugin);
				exit(1);
			}
	
		delete plugin;
    }

	if (inputFile == "") {
		fprintf(stderr, "\nUso: protomake [-j] [-c] [-h] -i file [-d out]\n\n");
		exit(1);
	}

	if ((yyin = fopen(inputFile, "r")) == NULL)
	{
		fprintf(stderr, "Erro ao tentar abrir o arquivo %s!\n", inputFile);
		exit(1);
	}

	if (yyparse())
	{
		fprintf(stderr, "No foi possvel compilar %s!\n", inputFile);
		exit(1);
	}

	TypeCheckerVisitor typeChecker;
	root->visit(typeChecker);

	if (generator & C) {
		fprintf(stderr, "Gerando cdigo para C...\n");
		CCodeGeneratorVisitor ccodeGenerator(baseName, baseDir);
		root->visit(ccodeGenerator);
	}

	if (generator & JAVA) {
		fprintf(stderr, "Gerando cdigo para Java...\n");
		JavaCodeGeneratorVisitor java(baseName, baseDir);
		root->visit(java);
	}

	fprintf(stderr, "Sintaxe de %s correta!\n", inputFile);
	fclose(yyin);

    if (etherInfo != NULL) {
        fprintf(stderr, "Gerando o plugin do ethereal para o protocolo...\n");
        generateEtherealPlugin(baseDir, &fiMap, etherInfo);
        fprintf(stderr, "Plugin gerado!\n");
    }

	free(baseDir);
	free(inputFile);
	free(baseName);
	return 0;
}
                                                                                                                                                           
int yyerror(char *msg)
{
	fprintf(stderr, "Line %d: %s\n", lines+1, msg);
    exit(1);
	
	return 0;
}

/** Generates an Ethereal plugin */
static void generateEtherealPlugin(char *baseDir,
								   map<string, vector<FieldInfo> > *fiMap,
                                   multimap<EtherType, EtherInfoType> *eit)
{
    multimap<EtherType, EtherInfoType>::iterator itr;

    /** Check if more than one or if no name/port/transport protocol was defined! */
    if (eit->count(NAME) > 1)
    {
        itr = eit->find(NAME);
        itr++;
        fprintf(stderr, "Line %d: More than one name was defined for the protocol!\n",
                (*itr).second.lines + 1);
        exit(1);
    }
    else if (eit->count(NAME) == 0)
    {
        fprintf(stderr, "No name was defined for the protocol!\n");
        exit(1);
    }
    if (eit->count(ABBREV) > 1)
    {
        itr = eit->find(ABBREV);
        itr++;
        fprintf(stderr, "Line %d: More than one short name was defined for the protocol!\n",
                (*itr).second.lines + 1);
        exit(1);
    }
    else if (eit->count(ABBREV) == 0)
    {
        fprintf(stderr, "No short name was defined for the protocol!\n");
        exit(1);
    }
    else if (eit->count(PORT) > 1)
    {
        itr = eit->find(PORT);
        itr++;
        fprintf(stderr, "Line %d: More than one port was defined for the protocol!\n",
                (*itr).second.lines + 1);
        exit(1);
    }
    else if (eit->count(PORT) == 0)
    {
        fprintf(stderr, "No port was defined for the protocol!\n");
        exit(1);
    }
    else if (eit->count(TRANSPORT) > 1)
    {
        itr = eit->find(TRANSPORT);
        itr++;
        fprintf(stderr, "Line %d: More than one transport protocol was defined for the protocol!\n",
                (*itr).second.lines + 1);
        exit(1);
    }
    else if (eit->count(TRANSPORT) == 0)
    {
        fprintf(stderr, "No transport protocol was defined for the protocol!\n");
        exit(1);
    }

    /** Create a file with the correct name to write the plugin */
    itr = eit->find(ABBREV);
    FILE *plugin;
    char fileName[2048];
    char *abbrev = ++((*itr).second.string);
    unsigned i;

    /**
     * This alters the element in the multimap. Any statement like this will.
     */
    abbrev[strlen(abbrev)-1] = '\0';
    for (i = 0; abbrev[i]; i++)
        abbrev[i] = tolower(abbrev[i]);
    

    sprintf(fileName, "%s/plugin/packet-%s.c", baseDir, (*itr).second.string);
    if ((plugin = fopen(fileName, "w")) == NULL)
    {
        fprintf(stderr, "Unable to create %s!\n", fileName);
        exit(1);
    }

    /** Write file header */
    writeHeader(plugin, fileName, eit);

    /** Declare necessary globals */
    int j;
    FieldInfo fi;
    fi.identifier = "code";
    fi.bitByte = FieldType::BIT;
    fi.size = NULL;
    
    j = 1;
    fprintf(plugin, "/** Message  structures. Inicialize the protocol and registered fields */\n");
    for (map<string, vector<FieldInfo> >::iterator itr = fiMap->begin();
         itr != fiMap->end();
         itr++, j++)
    {
        fi.init = new IntegerLiteral(NULL, j);
        ((*itr).second).push_back(fi);

        fprintf(plugin, "typedef struct {\n");
        for (i = 0; i < (*itr).second.size(); i++)
            fprintf(plugin, "\tint %s_%s;\n", abbrev, ((*itr).second)[i].identifier);
        fprintf(plugin, "} hf_%s_fields;\n", (*itr).first.c_str());
        fprintf(plugin, "static hf_%s_fields hf_%s = { ", (*itr).first.c_str(), (*itr).first.c_str());
        for (i = 0; i < (*itr).second.size(); i++)
        {
            if (!i) fprintf(plugin, "-1");
            else fprintf(plugin, ", -1");
        }
        fprintf(plugin, " };\n\n");
    } 
    fprintf(plugin, "static int proto_%s = -1;\n\n", abbrev);
    fprintf(plugin, "/* Initialize the subtree pointers */\n");
    fprintf(plugin, "static gint ett_%s = -1;\n\n", abbrev);

    /** Write protocol dissector */
    writeDissector(plugin, fiMap, eit);

    /** Write protocol register function */
    fprintf(plugin, "void proto_register_%s(void)\n{\n", abbrev);
    fprintf(plugin, "/** Setup list of header fields */\n");
    fprintf(plugin, "\t static hf_register_info hf[] = {\n");
    
    /** Setup the list of header fields */
    for (map<string, vector<FieldInfo> >::iterator itr = fiMap->begin();
         itr != fiMap->end();
         itr++)
    {
        if (itr != fiMap->begin())
            fprintf(plugin, ",\n");

        for (i = 0; i < (*itr).second.size(); i++)
        {
            if (i)
                fprintf(plugin, ",\n");
            fprintf(plugin, "\t\t{ &(hf_%s.%s_%s),\n", (*itr).first.c_str(), abbrev,
                                                       ((*itr).second)[i].identifier);
            fprintf(plugin, "\t\t\t{ \"%s\", \"%s.%s_%s\",\n", ((*itr).second)[i].identifier,
                                                               abbrev,
                                                               (*itr).first.c_str(),
                                                               ((*itr).second)[i].identifier);

            if ((*itr).second[i].bitByte == FieldType::BIT)
                fprintf(plugin, "\t\t\tFT_INT32, BASE_DEC, NULL, 0x0,\n\t\t\t\"%s\" }\n", ((*itr).second)[i].identifier);
            else
                fprintf(plugin, "\t\t\tFT_STRING, BASE_NONE, NULL, 0x0,\n\t\t\t\"%s\" }\n", ((*itr).second)[i].identifier);
            fprintf(plugin, "\t\t}");
        }
    }
    fprintf(plugin, "\n\t};\n\n");

    fprintf(plugin, "/** Setup protocol subtree array */\n");
    fprintf(plugin, "\tstatic gint *ett[] = { &ett_%s };\n\n", abbrev);
   
    fprintf(plugin, "/** Register the protocol name and description */\n");
    char *ABBREV = strdup(abbrev);
    for (i = 0; ABBREV[i]; i++)
        ABBREV[i] = toupper(ABBREV[i]);

    char *name = (*(eit->find(NAME))).second.string;
    fprintf(plugin, "\tproto_%s = proto_register_protocol(\"%s\",\n"
                    "\t                                      \"%s\",\n"
                    "\t                                      \"%s\");\n\n",
                    abbrev, name, ABBREV, abbrev);

    fprintf(plugin, "/** Required function calls to register the header fields and subtrees used */\n");
    fprintf(plugin, "\tproto_register_field_array(proto_%s, hf, array_length(hf));\n", abbrev);
    fprintf(plugin, "\tproto_register_subtree_array(ett, array_length(ett));\n}");

    /** Write rest of file */
    writeRest(plugin, (*(eit->find(TRANSPORT))).second.string, (*(eit->find(PORT))).second.string, abbrev);
    fclose(plugin);

	sprintf(fileName, "%s/plugin/Makefile.am", baseDir);
	FILE *makefile = fopen(fileName, "w");
	if (makefile)
	{
		fprintf(makefile, "INCLUDES = -I$(top_src_dir)\n\n");

		fprintf(makefile, "plugindir = @plugindir@\n\n");

		fprintf(makefile, "plugin_LTLIBRARIES = %s.la\n", abbrev);
		fprintf(makefile, "%s_la_SOURCES = packet-%s.c\n", abbrev, abbrev);
		fprintf(makefile, "%s_la_LDFLAGS = -module -avoid-version\n\n", abbrev);

		fprintf(makefile, "LIB = \n\n");

		fprintf(makefile, "%s_la_DEPENDENCIES = packet-%s-static.o\n\n", abbrev, abbrev);

		fprintf(makefile, "packet-%s-static.o: packet-%s.c\n", abbrev, abbrev);
		fprintf(makefile, "\t$(LTCOMPILE) -c -o packet-%s-static.o -D__ETHEREAL_STATIC__ $(srcdir)/packet-%s.c\n",
						  abbrev, abbrev);
						  
		fprintf(makefile, "CLEANFILES = %s \n", abbrev);
		fprintf(makefile, "EXTRA_DIST = Makefile.nmake\n");
	}
	else fprintf(stderr, "Unable to create Makfile.am!\n");
	fclose(makefile);

	sprintf(fileName, "%s/plugin/Makefile.nmake", baseDir);
	makefile = fopen(fileName, "w");
	if (makefile)
	{
		fprintf(makefile, "include ..\\..\\config.nmake\n\n");

		fprintf(makefile, "CFLAGS=/DHAVE_CONFIG_H /I../.. /I../../wiretap \\\n");
		fprintf(makefile, "\t/I$(GLIB_DIR) /I$(GTK_DIR) /I$(GLIB_DIR)/gmodule \\\n");
		fprintf(makefile, "/I$(PCAP_DIR)\\include $(LOCAL_CFLAGS)\n\n");
		
		fprintf(makefile, "OBJECTS=packet-%s.obj\n\n", abbrev);
		fprintf(makefile, "%s.dll %s.exp %s.lib : packet-%s.obj ..\\plugin_api.obj\n",
						  abbrev, abbrev, abbrev, abbrev);
		fprintf(makefile, "\tlink -dll /out:%s.dll packet-%s.obj ..\\plugin_api.obj \\\n",
						  abbrev, abbrev);
		fprintf(makefile, "$(GLIB_DIR)\\glib-$(GLIB_VERSION).lib\n\n");
		fprintf(makefile, "clean:\n\trm -f $(OBJECTS) %s.dll %s.exp %s.lib\n",
						  abbrev, abbrev, abbrev);
	}
	else fprintf(stderr, "Unable to create Makefile.nmake!\n");
	fclose(makefile);
}

static void writeHeader(FILE *plugin, char *fileName,
                        multimap<EtherType, EtherInfoType> *eit)
{
    int total;
    char *author, *email;
    char *name = ++((*(eit->find(NAME))).second.string);
    multimap<EtherType, EtherInfoType>::iterator itr;

    name[strlen(name)-1] = '\0';
    fprintf(plugin, "/* %s\n", &fileName[2]);
    fprintf(plugin, " * Routines for %s dissection\n", name);

    fprintf(plugin, " * Copyright 2003");
    for (total = eit->count(AUTHOR), itr = eit->find(AUTHOR);
         total > 0;
         itr++, total--)
    {
        author = ++((*itr).second.string);

        author[strlen(author) - 1] = '\0';
        fprintf(plugin, ", %s", author);
    }

    fprintf(plugin, " <");
    for (total = eit->count(EMAIL), itr = eit->find(EMAIL);
         total > 0;
         itr++, total--)
    {
        email = ++((*itr).second.string);

        email[strlen(email) - 1] =  '\0';
        fprintf(plugin, "%s, ", email);
    }
    fprintf(plugin, ">\n");

    char *header = " * \n\
 * Ethereal - Network traffic analyzer\n\
 * By Gerald Combs <gerald@ethereal.com>\n\
 * Copyright 1998 Gerald Combs\n\
 * \n\
 * Generated from ProtoMake by Igor Canana, Fbio Guerra, Rodrigo Arajo &\n\
 * Thiago Souto Maior, Copyright 2003.\n\
 *\n\
 * This program is free software; you can redistribute it and/or\n\
 * modify it under the terms of the GNU General Public License\n\
 * as published by the Free Software Foundation; either version 2\n\
 * of the License, or (at your option) any later version.\n\
 *\n\
 * This program is distributed in the hope that it will be useful,\n\
 * but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
 * GNU General Public License for more details.\n\
 *\n\
 * You should have received a copy of the GNU General Public License\n\
 * along with this program; if not, write to the Free Software\n\
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n\
 */\n\
\n\
#ifdef HAVE_CONFIG_H\n\
# include \"config.h\"\n\
#endif\n\
\n\
#include <stdio.h>\n\
#include <stdlib.h>\n\
#include <string.h>\n\
\n\
/* Includes glib.h! */\n\
#include <gmodule.h>\n\
\n\
#ifdef NEED_SNPRINTF_H\n\
# include \"snprintf.h\"\n\
#endif\n\
\n\
#include <epan/packet.h>\n\
#include \"plugins/plugin_api.h\"\n\
#include \"plugins/plugin_api_defs.h\"\n\
\n\
#ifndef __ETHEREAL_STATIC__\n\
G_MODULE_EXPORT const gchar version[] = \"1.0\";\n\
#endif\n\n";

    fprintf(plugin, "%s", header);
}


static void writeDissector(FILE *plugin, map<string, vector<FieldInfo> > *fiMap,
                           multimap<EtherType, EtherInfoType> *eit)
{
    char *abbrev = (*(eit->find(ABBREV))).second.string;
    char *ABBREV = strdup(abbrev);
    unsigned i;

    for (i = 0; ABBREV[i]; i++)
        ABBREV[i] = toupper(ABBREV[i]);

    fprintf(plugin, "/** Dissector main code! */\n");
    fprintf(plugin, "void dissect_%s(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)\n{\n", abbrev);

    fprintf(plugin, "/** Set up structures needed to add the protocol subtree and manage it */\n");
    fprintf(plugin, "\tproto_item *tree_item;\n\tproto_tree *%s_tree;\n\tguint16 code;\n", abbrev);

    for (map<string, vector<FieldInfo> >::iterator itr = fiMap->begin();
         itr != fiMap->end();
         itr++)
    {
        fprintf(plugin, "\tstatic struct {\n");
        for (i = 0; i < (*itr).second.size(); i++)
        {
            if (((*itr).second)[i].bitByte == FieldType::BIT)
                fprintf(plugin, "\t\tguint32 ");
            else
                fprintf(plugin, "\t\tguint8 *");
            fprintf(plugin, "%s_%s;\n", abbrev, ((*itr).second)[i].identifier);            
        }
        fprintf(plugin, "\t} msg_%s;\n\n", (*itr).first.c_str());
    }

    fprintf(plugin, "/** Make entries in Protocol column and Info column on summary display */\n");
    fprintf(plugin, "\tif (check_col(pinfo->cinfo, COL_PROTOCOL))\n");
    fprintf(plugin, "\t\tcol_set_str(pinfo->cinfo, COL_PROTOCOL, \"%s\");\n\n", ABBREV);

    fprintf(plugin, "/** Verify the message sent and dissect it properly */\n");
    fprintf(plugin, "\tif (tree) {\n");
    fprintf(plugin, "\t\tcode = tvb_get_ntohs(tvb, 0);\n\n");

    fprintf(plugin, "\t\ttree_item = proto_tree_add_item(tree, proto_%s, tvb, 0, -1, FALSE);\n", abbrev);
    fprintf(plugin, "\t\t%s_tree = proto_item_add_subtree(tree_item, ett_%s);\n\n", abbrev, abbrev);

    fprintf(plugin, "\t\tif (");

    for (map<string, vector<FieldInfo> >::iterator itr = fiMap->begin();
         itr != fiMap->end();
         itr++)
    {
        if (itr != fiMap->begin())
            fprintf(plugin, "\t\telse if (");
        vector <FieldInfo> temp = ((*itr).second);
        IntegerLiteral *msgCode = dynamic_cast<IntegerLiteral*> (temp[temp.size() - 1].init);
        fprintf(plugin, "code == %d)\n\t\t{\n", msgCode->getValue());
        fprintf(plugin, "\t\t\tint offset = 2; //sizeof(code) == 2 since code is a guint16 :)\n\n");

        /** Really dissect the message now! \_:)_/ */
        for (i = 0; i < (*itr).second.size() - 1; i++)
        {
            FieldInfo fi = (*itr).second[i];

            if (fi.bitByte == FieldType::BIT)
            {
                int value = (dynamic_cast<IntegerLiteral*> (fi.size))->getValue();
                
                if (value <= 8)
                {
                    fprintf(plugin, "\t\t\tmsg_%s.%s_%s = tvb_get_guint8(tvb, offset);\n",
                            (*itr).first.c_str(), abbrev, fi.identifier);
					fprintf(plugin, "\t\t\tproto_tree_add_item(%s_tree, hf_%s.%s_%s, tvb, offset, 1, FALSE);\n",
							abbrev, (*itr).first.c_str(), abbrev, fi.identifier);
                    fprintf(plugin, "\t\t\toffset++;\n\n");
                }
                else if (value <= 16)
                {
                    fprintf(plugin, "\t\t\tmsg_%s.%s_%s = tvb_get_ntohs(tvb, offset);\n",
                            (*itr).first.c_str(), abbrev, fi.identifier);
					fprintf(plugin, "\t\t\tproto_tree_add_item(%s_tree, hf_%s.%s_%s, tvb, offset, 2, FALSE);\n",
							abbrev, (*itr).first.c_str(), abbrev, fi.identifier);
                    fprintf(plugin, "\t\t\toffset += 2;\n\n"); 
                } 
                else if (value <= 32 || value > 16) // This might be a problem for > 32 bits!
                {
                    fprintf(plugin, "\t\t\tmsg_%s.%s_%s = tvb_get_ntohl(tvb, offset);\n",
                            (*itr).first.c_str(), abbrev, fi.identifier);
					fprintf(plugin, "\t\t\tproto_tree_add_item(%s_tree, hf_%s.%s_%s, tvb, offset, 4, FALSE);\n",
							abbrev, (*itr).first.c_str(), abbrev, fi.identifier);
                    fprintf(plugin, "\t\t\toffset += 4;\n\n");
                }
            }
            else if (fi.bitByte == FieldType::BYTE &&
                     typeid(*(fi.size)) == typeid(IntegerLiteral))
            {
                int value = (dynamic_cast<IntegerLiteral*> (fi.size))->getValue();

                fprintf(plugin, "\t\t\tmsg_%s.%s_%s = tvb_memdup(tvb, offset, %d);\n",
                        (*itr).first.c_str(), abbrev, fi.identifier, value);
				fprintf(plugin, "\t\t\tproto_tree_add_string(%s_tree, hf_%s.%s_%s, tvb, offset, %d, msg_%s.%s_%s);\n",
						abbrev, (*itr).first.c_str(), abbrev, fi.identifier, value, (*itr).first.c_str(), abbrev, fi.identifier);
				fprintf(plugin, "\t\t\toffset += %d;\n\n", value);
            }
            else if (fi.bitByte == FieldType::BYTE &&
                     typeid(*(fi.size)) == typeid(Identifier))
            {
                const char *name = (dynamic_cast<Identifier*>(fi.size))->getName().c_str();

                fprintf(plugin, "\t\t\tmsg_%s.%s_%s = tvb_memdup(tvb, offset, msg_%s.%s_%s);\n",
                        (*itr).first.c_str(), abbrev, fi.identifier, (*itr).first.c_str(),
                        abbrev, name);
				fprintf(plugin, "\t\t\tproto_tree_add_string(%s_tree, hf_%s.%s_%s, tvb, offset, msg_%s.%s_%s, msg_%s.%s_%s);\n",
						abbrev, (*itr).first.c_str(), abbrev, fi.identifier, (*itr).first.c_str(), abbrev, name,
						(*itr).first.c_str(), abbrev, fi.identifier);
                fprintf(plugin, "\t\t\toffset += msg_%s.%s_%s;\n\n", (*itr).first.c_str(), abbrev, name);
            }
        }

        fprintf(plugin, "\t\t}\n");
    }    
    fprintf(plugin, "\t}\n");
    fprintf(plugin, "}\n\n");
}


static void writeRest(FILE *plugin, char *trans, char *port, char *abbrev)
{
    char *rest = "\n\n/* If this dissector uses sub-dissector registration add a registration routine.\n\
   This format is required because a script is used to find these routines and\n\
   create the code that calls these routines.\n\
*/\n";

    fprintf(plugin, "%s", rest);
    fprintf(plugin, "void proto_reg_handoff_%s(void)\n{\n", abbrev);
	fprintf(plugin, "\tdissector_handle_t %s_handle;\n\n", abbrev);
	
    fprintf(plugin, "\t%s_handle = create_dissector_handle(dissect_%s,\n\t\tproto_%s);\n", abbrev, abbrev, abbrev);
    fprintf(plugin, "	dissector_add(\"%s.port\", %s, %s_handle);\n", trans, port, abbrev);

    rest = "}\n\
\n\
#ifndef __ETHEREAL_STATIC__\n\
G_MODULE_EXPORT void plugin_init(plugin_address_table_t *pat)\n\
{\n\
	/* initialise the table of pointers needed in Win32 DLLs */\n\
	plugin_address_table_init(pat);\n\
\n\
	/* register the new protocol, protocol fields, and subtrees */\n";
    fprintf(plugin, "%s", rest);
	fprintf(plugin, "\tif (proto_%s == -1) /* execute protocol initialization only once */\n", abbrev);
	fprintf(plugin, "\t\tproto_register_%s();\n}\n", abbrev);
    rest = "#endif\n\
\n\
#ifndef __ETHEREAL_STATIC__\n\
G_MODULE_EXPORT void plugin_reg_handoff(void)\n\
{\n\
	proto_reg_handoff_";
    fprintf(plugin, "%s%s();\n}\n#endif\n", rest, abbrev);
}
